@@ -1,56 +1,64 @@
 #
-# Spider, get web page, extract word from the content.
+# Spider, get web page, extract word from the content, download file.
 #
 # ::
 #     # Spider usage examples:
 #
 #     from net import Spider
 #
+#
 #     # create a Spider object for specified url
 #     url = "https://www.python.org/"
 #     spider = Spider(url)
 #
-#     # get the web-page content
-#     spider = spider.get()
 #
-#     # You can user find(Spider.XXX) to find certain content
-#     # Notice: the result is not always correct because of the structure of HTML source code
+#     # You can user Spider.find_xxx() to find information in the web page
 #
-#     # find the links on the page
-#     link_urls = spider.find(Spider.LINKS)
+#     # find all links in the page
+#     link_urls = spider.find_links()
 #     print('links:', link_urls)
 #
-#     # find the url of the images on the page
-#     img_urls = spider.find(Spider.IMAGES)
+#     # find the links after 'Latest News'
+#     link_urls = spider.find_links('Latest News')
+#     print('links:', link_urls)
+#
+#
+#     # find the links 'Latest News' and the link's url contains "blog."
+#     link_urls = spider.find_links('Latest News', contains="blog.")
+#     print('links:', link_urls)
+#
+#
+#     # find the urls of the images in the page
+#     img_urls = spider.find_images()
 #     print('images:', img_urls)
 #
 #     if len(img_urls) > 0:
 #         # create a new spider to download the first image url, save image to filename 'pic.xxx'
 #         # (file extension will be added automatically)
 #         filename = Spider(url, img_urls[0]).download("pic")
-#
-#         # open image file using PIL
-#         # from PIL import Image
-#         # Image.open(filename).show()
-#
-#
-#     # find the list items after 'Latest News'
-#     words = spider.find(Spider.LIST_ITEMS, 'Latest News')
-#     print('Latest News:', words)
+#         print('file %s downloaded' % repr(filename))
 #
 #     print("-----------------------------------")
 #
-#     # find the codes
-#     codes = spider.find(Spider.CODES)
+#     # find the codes in the page
+#     codes = spider.find_codes()
 #     if codes:
 #         print('code:\n', codes[0])
 #
-#     print("====================================")
+#     print("-----------------------------------")
+#
+#     # find the list items after 'Latest News'
+#     words = spider.find_list_items('Latest News')
+#     print('Latest News:', words)
 #
 #     # find the text of the paragraph after 'Download'
-#     text = spider.find(Spider.PARAGRAPH, 'Download')
+#     text = spider.find_paragraph('Download')
 #     print('Download paragraph: ', text)
 #
+#     tables = Spider("https://www.w3school.com.cn/tags/tag_table.asp").find_tables(text_only=True)
+#     if len(tables) > 0:
+#         print('table 0:', tables[0].to_list())
+#
 #
 #     # Advanced find example
 #     #
@@ -82,6 +90,10 @@
 #     print("menus2:", words_list)  # words_list will be a list, each item is a list of two words
 #     # the result is  [['/', 'Python'], ['/psf-landing/', 'PSF'], ...
 #
+#
+#
+#
+
 
 from html.parser import HTMLParser
 import re
@@ -89,7 +101,7 @@ import json
 import os
 
 from .util import StrUtil
-from .http import Http, Url
+from .http import Http, Url, UserAgent
 
 
 # A class to parse HTML to text
@@ -102,6 +114,8 @@ class DeHTMLParser(HTMLParser):
         HTMLParser.__init__(self)
         self._text = []
         self._last_tag = None
+        self.tag = kwargs.get('tag', None)
+        self.tag_attrs = {}
         self.line_break = kwargs.get('line_break', False)
 
     def handle_data(self, data):
@@ -119,6 +133,9 @@ class DeHTMLParser(HTMLParser):
     def handle_starttag(self, tag, attrs):
         # print(tag, attrs)
         self._last_tag = tag
+        if self.tag and tag == self.tag:
+            for attr in attrs:
+                self.tag_attrs[attr[0]] = attr[1]
         if tag == 'p':
             self._text.append('\n\n')
         elif tag == 'br':
@@ -139,6 +156,54 @@ class DeHTMLParser(HTMLParser):
         return s.join(self._text).strip()
 
 
+# A class to parse HTML form
+class HtmlForm(HTMLParser):
+    """
+    A class to parse HTML to text
+
+    """
+
+    class Input(dict):
+        def __init__(self, attrs):
+            for attr in attrs:
+                self[attr[0]] = attr[1]
+
+    def __init__(self, html_text, **kwargs):
+        HTMLParser.__init__(self)
+        self.attrs = {}
+        self.inputs = []
+        if html_text:
+            self.feed(html_text)
+
+    def handle_data(self, data):
+        pass
+        # if self._last_tag == "script":
+        #     return
+        #
+        # text = data.strip()
+        # if len(text) > 0:
+        #     if self.line_break:
+        #         text = re.sub('[ \t]+', ' ', text)
+        #     else:
+        #         text = re.sub('[ \t\r\n]+', ' ', text)
+        #     self._text.append(text + ' ')
+
+    def handle_starttag(self, tag, attrs):
+        # print(tag, attrs)
+        if tag == 'input':
+            self.inputs.append(HtmlForm.Input(attrs))
+
+    def handle_startendtag(self, tag, attrs):
+        pass
+
+    def text(self):
+        if self.line_break:
+            s = '\n'
+        else:
+            s = ''
+        return s.join(self._text).strip()
+
+
 # convert html to text
 def html2text(html, convert=True, line_break=False):
     """
@@ -184,8 +249,28 @@ class Table(list):
         pass
 
     class Cell:
+        def __init__(self, text, urls=None):
+            self.text = text
+            self.urls = urls or []
+            self.attrs = {}
+
+        def __repr__(self):
+            return '<Cell %s>' % repr(self.text)
+
+    def _regulate(self):
+        # TODO: process span row, span col
         pass
 
+    def to_list(self):
+        self._regulate()
+        result = []
+        for row in self:
+            r = []
+            for cell in row:
+                r.append(cell.text)
+            result.append(r)
+        return result
+
     def __repr__(self):
         result = '[\n'
         count = 0
@@ -223,24 +308,36 @@ class Spider:
 
     def __init__(self, url: str = None, sub_url: str = None):
         self.pure_text = True  # whether return pure text
-
+        self._user_agent = UserAgent.chrome
         self._url = Url.join(url, sub_url)
         self._page = None
 
         if url is None:
             self._url = Spider.BING
 
-    def search(self, word):
+    def user_agent(self, keyword):
+        keyword = str(keyword).strip().lower()
+        if keyword == 'android':
+            self._user_agent = UserAgent.android
+        elif keyword == 'ios':
+            self._user_agent = UserAgent.ios
+        elif keyword == 'iphone':
+            self._user_agent = UserAgent.ios
+        elif keyword == 'chrome':
+            self._user_agent = UserAgent.chrome
+        elif keyword == 'ie':
+            self._user_agent = UserAgent.ie
+        else:
+            raise ValueError("unknown UserAgent keyword %s" % repr(keyword))
+        return self
+
+    def search(self, word=None):
         """ search a word """
         self._page = Http.get(self._url, word).text
         return self
 
-    def get(self, word=''):
-        """ get word """
-        self._page = Http.get(self._url, word).text
-        return self
-
     def download(self, filename):
+        """ download and save to file """
         return Http.get(self._url, '').save_to_file(filename)
 
     # noinspection PyMethodMayBeStatic
@@ -262,14 +359,14 @@ class Spider:
     def _ensure_page(self):
         """ ensure self._page has content """
         if self._url and not self._page:
-            self.get()
+            self._page = Http.get(self._url).text
 
     def find(self, before, after=None, begin=None, end=None):
         """
         find information on the web page
 
         :param before: texts which is before the finding text.  could be a string or a list of string.
-        :param after: (optional) texts which is before the finding text, could be a string or a list of string
+        :param after: (optional) texts which is after the finding text, could be a string or a list of string
         :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
         :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
         :return:  return string
@@ -291,14 +388,23 @@ class Spider:
         s = StrUtil.get_word(self._page, before, after, start)
         return html2text(s, self.pure_text)
 
+    def find_pairs(self, begin, pair='{}'):
+        self._ensure_page()
+
+        if not self._page:
+            return None
+
+        s = StrUtil.find_pair(self._page, begin, pair=pair)
+        return s
+
     def find_links(self, begin=None, end=None, contains=None, regex=False):
         """
         find links in the webpage. A link is a tag like '<a href="url">text</a>' in HTML.
 
         :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
         :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
-        :param contains: (optional) url style to filter the links
-        :param regex: (optional) whether url is a regex pattern
+        :param contains: (optional) the substring that the link's url should contains
+        :param regex: (optional) whether contains is a regex pattern
         :return: return a list, each item of the list is a tuple of (url, text) of a link.
         """
         self._ensure_page()
@@ -327,18 +433,33 @@ class Spider:
         else:
             return links
 
-    def find_images(self, begin=None, end=None):
+    def find_images(self, begin=None, end=None, contains=None, regex=False):
         """
         find images in the webpage. An image is a tag like '<img src="url">' in HTML.
 
         :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
         :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
+        :param contains: (optional) the substring that the image's url should contains
+        :param regex: (optional) whether contains is a regex pattern
         :return: return a list, each item of the list is url of image.
         """
         before = ['<img', 'src="']
         after = '"'
         betweens = [[before, after]]
-        return self.find_words_list(betweens, begin, end)
+        urls = self.find_words_list(betweens, begin, end)
+
+        # filter the links
+        if contains:
+            result = []
+            for url in urls:
+                if regex:
+                    if re.search(contains, url) is not None:
+                        result.append(url)
+                elif url.find(contains) >= 0:
+                    result.append(url)
+            return result
+        else:
+            return urls
 
     def find_codes(self, begin=None, end=None):
         """
@@ -376,11 +497,14 @@ class Spider:
 
     def find_forms(self, begin=None, end=None):
         self._ensure_page()
-        s = self._page
-        # print(s)
-        forms = self._get_texts(self._page, '<form', '</form', begin, end)
-        print(forms)
-        pass
+        text = self._page
+
+        forms = self._get_texts(text, '<form', '</form', begin, end)
+
+        result = []
+        for form in forms:
+            result.append(HtmlForm(form))
+        return result
 
     def set_text(self, text):
         """ set text of the web page """
@@ -419,7 +543,7 @@ class Spider:
             # find row in '<tr> ... </tr>'
             rows = self._get_texts(table, '<tr', '</tr')
             for row in rows:
-                row_obj = []
+                row_obj = Table.Row()
                 # find cell in '<td> .. </td>'
                 if row.find('<th') >= 0:
                     cells = self._get_texts(row, '<th', '</th')
@@ -432,11 +556,14 @@ class Spider:
                     if cell.find('<a') >= 0:
                         urls = Spider().set_text(cell).find_links()
                     # get cell text
-                    cell_text = html2text(cell, line_break=True)
+                    parser = DeHTMLParser(tag='td', line_break=True)
+                    parser.feed(cell)
+                    cell_text = parser.text()
                     if text_only:
-                        cell_obj = cell_text
+                        cell_obj = Table.Cell(cell_text)
                     else:
-                        cell_obj = (cell_text, urls)
+                        cell_obj = Table.Cell(cell_text, urls)
+                    cell_obj.attrs = parser.tag_attrs
                     # append cell to the row
                     row_obj.append(cell_obj)
                 # append row to the table
@@ -465,7 +592,7 @@ class Spider:
 
         return b
 
-    def find_list_items(self, begin=None, end=None, is_ol=False):
+    def find_list_items(self, begin=None, is_ol=False):
         """
         find list items in the webpage. list items are within '<ul> <li></li> </ul>' in HTML
 
@@ -496,7 +623,7 @@ class Spider:
         find words
 
         :param before: texts which is before the finding text.  could be a string or a list of string.
-        :param after: (optional) texts which is before the finding text, could be a string or a list of string
+        :param after: (optional) texts which is after the finding text, could be a string or a list of string
         :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
         :return:  return string
         """
@@ -525,7 +652,7 @@ class Spider:
         find a list of word
 
         :param before: texts which is before the finding text.  could be a string or a list of string.
-        :param after: (optional) texts which is before the finding text, could be a string or a list of string
+        :param after: (optional) texts which is after the finding text, could be a string or a list of string
         :param begin: (optional) texts from where to begin the finding, could be a string or a list of string or an offset int
         :param end: (optional) texts from where to end the finding, could be a string or a list of string or an offset int
         :return:  return string
@@ -580,6 +707,11 @@ class Spider:
 
             return words
 
+    @property
+    def html(self):
+        self._ensure_page()
+        return self._page
+
     @property
     def text(self):
         self._ensure_page()
