@@ -185,9 +185,9 @@ class StrUtil:
         :Chinese: 从文本中提取一个词，该词在查找条件before后, 查找条件after前
 
         :param text:    the string
-        :param before:  the before condition, could be a sub string or a list of string.<br>
-                :Chinese: 查找条件before
-        :param after:   the end condition, could be a sub string or a list of string.<br>
+        :param before:   the texts which is before the finding word, could be a string or a list of string.<br>
+                :Chinese: 查找在 after 之后的词
+        :param after:   the texts which is after the finding word,, could be a sub string or a list of string.<br>
                 :Chinese: 查找条件end
         :param begin:    offset to start finding.
                 :Chinese:（可选）开始查找的位置
@@ -221,14 +221,14 @@ class StrUtil:
                     return text[b + len1: e], e + len2
                 else:
                     return text[b + len1: e]
+
+        if return_offset:
+            return None, begin
         else:
-            if return_offset:
-                return None, begin
-            else:
-                return None
+            return None
 
     @staticmethod
-    def get_word_list(text, begin, betweens, start_offset=0, end=None):
+    def get_word_list(text, begin, betweens, end=None, start_offset=0):
         """
         get a list of keywords from the text.
 
@@ -251,13 +251,14 @@ class StrUtil:
                          每一个元素单词定义, 一个 2个元素tuple, 第0个元素是before条件，
                          第1个元素是after条件.
 
-        :param start_offset: (optional) offset to start finding
         :param end:  (optional) end condition, could be a sub string or a list of string.<br>
+        :param start_offset: (optional) offset to start finding
         :return:  return a list of words
         """
         result = []
         if begin:
             offset, length = StrUtil.match_offset_length(text, begin, start_offset)
+            offset = offset + length
         else:
             offset = 0
 
@@ -295,6 +296,7 @@ class StrUtil:
         else:
             return result
 
+
     @staticmethod
     def is_json_string(s):
         """
@@ -354,6 +356,60 @@ class StrUtil:
                     count += 1
         return None
 
+    @staticmethod
+    def find_pair(text, begin=None, return_offset=False, pair='{}'):
+        if begin is None:
+            begin = 0
+
+        if begin is not None and not isinstance(begin, int):
+            offset, length = StrUtil.match_offset_length(text, begin, 0)
+            if offset < 0:
+                return None, None if return_offset else None
+            else:
+                begin = offset + length
+
+        if begin < 0:
+            if return_offset:
+                return None, begin
+            else:
+                return None
+
+        start_char = '{'
+        end_char = '}'
+        if len(pair) == 1:
+            start_char = end_char = pair
+        elif len(pair) >= 2:
+            start_char = pair[:1]
+            end_char = pair[1:2]
+
+        count = 0
+        length = len(text)
+        offset = begin
+        while offset < length:
+            c = text[offset]
+            offset += 1
+            if start_char == end_char:
+                if c == start_char:
+                    if count == 0:
+                        count += 1
+                    else:
+                        count -= 1
+                        if count <= 0:
+                            break
+            else:
+                if c == start_char:
+                    count += 1
+                elif c == end_char:
+                    count -= 1
+                    if count <= 0:
+                        break
+        ret = text[begin:offset]
+        if return_offset:
+            return ret, offset
+        else:
+            return ret
+
+
     @staticmethod
     def find_sep(text, sep, start_offset=0):
         """
